{
TfrmCalculatpr: part of TCalcEdit component.


Author: Giuseppe Madaffari
Copyright:  2000 by Giuseppe Madaffari. All rights reserved.
Created: 10/11/2000
Modified: 12/02/2001
Version: 1.0
License: you can freely use and distribute the included code
         for any purpouse, but you cannot remove this copyright
         notice. Send me any comment and update, they are really
         appreciated. :-)
Contact: gmadaffari@infomedia.it
}

unit Calc;

interface

uses
  SysUtils, Types, Classes, QGraphics, QControls, QForms, QDialogs,
  QStdCtrls, QExtCtrls, QButtons;

type
  TOperator = (opNull, opSum, opSubtract, opDivide, opMultiply);
  TfrmCalculator = class(TForm)
    Panel1: TPanel;
    txtDisplay: TEdit;
    btn7: TButton;
    btn9: TButton;
    btn8: TButton;
    btn4: TButton;
    btn5: TButton;
    btn6: TButton;
    btn1: TButton;
    btn2: TButton;
    btnMul: TButton;
    btnDiv: TButton;
    btn3: TButton;
    btn0: TButton;
    btnSum: TButton;
    btnSub: TButton;
    btnSign: TButton;
    btnDecSec: TButton;
    btnConvTo: TButton;
    btnConvFrom: TButton;
    btnEquals: TButton;
    btnBack: TButton;
    btnClear: TButton;
    procedure btnConvToClick(Sender: TObject);
    procedure btn1Click(Sender: TObject);
    procedure Button13Click(Sender: TObject);
    procedure btnDivClick(Sender: TObject);
    procedure btnEqualsClick(Sender: TObject);
    procedure btnBackClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure btnClearClick(Sender: TObject);
    procedure btnDecSecClick(Sender: TObject);
    procedure btnConvFromClick(Sender: TObject);
    procedure btnSignClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    FEdit: TEdit;
    Operator: TOperator;
    Operand: Extended;
    CommaOn: Boolean;
    ResetDisplay: Boolean;
    ConvRatio: Single;
    DecimalDigits: Integer;
    function GetValue: String;
    procedure SetValue(const Value: String);
  public
    { Public declarations }
    constructor Create(Edit: TEdit; ConversionRatio: Single = 0;
      DecDigits: Integer = 2; CurSimbol: string = 'Eur'); reintroduce;
    property Value: String read GetValue write SetValue;
  end;

var
  frmCalculator: TfrmCalculator;

implementation
uses QT;

{$R *.xfm}

procedure TfrmCalculator.btnConvToClick(Sender: TObject);
var
  Tmp: Extended;
begin
  Tmp := StrToFloat(txtDisplay.Text);
  Tmp := Tmp / ConvRatio;
  txtDisplay.Text := Format('%.' + IntToStr(DecimalDigits) + 'f', [Tmp]);
end;

procedure TfrmCalculator.btn1Click(Sender: TObject);
var
  S: String;
  Value: Extended;
begin
  if ResetDisplay then
  begin
    ResetDisplay := False;
    txtDisplay.Text := (Sender as TButton).Caption;
  end
  else if Length(txtDisplay.Text) < 11 then
  begin  // max 99999999999
    S := txtDisplay.Text;
    if CommaOn and (Pos(DecimalSeparator, S) = 0) then
      S := S + DecimalSeparator;
    S :=  S + (Sender as TButton).Caption;
    Value := StrToFloat(S);
    txtDisplay.Text := FloatToStr(Value);
  end;
end;

procedure TfrmCalculator.Button13Click(Sender: TObject);
begin
  txtDisplay.Text := '0';
  Operator := opNull;
end;

procedure TfrmCalculator.btnDivClick(Sender: TObject);
begin
  case (Sender as TButton).Caption[1] of
    '/': Operator := opDivide;
    '*': Operator := opMultiply;
    '+': Operator := opSum;
    '-': Operator := opSubtract;
    else
      Operator := opNull;
  end;
  Operand := StrToFloat(txtDisplay.Text);
  ResetDisplay := True;
  CommaOn := False;
end;

procedure TfrmCalculator.btnEqualsClick(Sender: TObject);
var
  Op2, Result: Extended;
begin
  if Operator = opNull then Exit;
  Op2 := StrToFloat(txtDisplay.Text);
  Result := 0;
  try
    case Operator of
      opDivide: Result := Operand / Op2;
      opMultiply: Result := Operand * Op2;
      opSum: Result := Operand + Op2;
      opSubtract: Result := Operand - Op2;
    end;
  except
    // do nothing, just catch the exception
  end;
  Operator := opNull;
  Operand := 0;
  CommaOn := False;
  ResetDisplay := True;
  txtDisplay.Text := FloatToStr(Result);
end;

procedure TfrmCalculator.btnBackClick(Sender: TObject);
var
  S: String;
begin
  S := Copy(txtDisplay.Text, 1, Length(txtDisplay.Text) - 1);
  if S = '' then S := '0';
  txtDisplay.Text := S;
end;

procedure TfrmCalculator.FormKeyPress(Sender: TObject; var Key: Char);
begin
  case Key of
    '0'..'9':
      TButton(FindComponent('btn'+Key)).Click;
    #8: btnBack.Click;
    #27: Close;
    'c','C': btnClear.Click;
    '+': btnSum.Click;
    '/': btnDiv.Click;
    '-': btnSub.Click;
    '*': btnMul.Click;
    '.', ',': btnDecSec.Click;
    #13, '=': btnEquals.Click;
  end;
  //Caption := IntToStr(Ord(key));
end;

procedure TfrmCalculator.btnClearClick(Sender: TObject);
begin
  Operator := opNull;
  CommaOn := False;
  txtDisplay.Text := '0';
end;

procedure TfrmCalculator.btnDecSecClick(Sender: TObject);
begin
  CommaOn := True;
end;

constructor TfrmCalculator.Create(Edit: TEdit; ConversionRatio: Single = 0;
  DecDigits: Integer  = 2; CurSimbol: string = 'Eur');
begin
  inherited Create(Edit);
  FEdit := Edit;
  ConvRatio := ConversionRatio;
  DecimalDigits := DecDigits;
  btnConvTo.Caption := '-> ' + CurSimbol;
  btnConvFrom.Caption := CurSimbol + ' ->';
  if ConvRatio = 0 then
  begin
    btnConvTo.Enabled := False;
    btnConvFrom.Enabled := False;
  end;
  Width := Panel1.Width;
  Height := Panel1.Height;
  FormStyle := fsNormal;
  BorderStyle := fbsNone;
end;

procedure TfrmCalculator.btnConvFromClick(Sender: TObject);
var
  Tmp: Extended;
begin
  Tmp := StrToFloat(txtDisplay.Text);
  Tmp := Tmp * ConvRatio;
  txtDisplay.Text := Format('%.' + IntToStr(DecimalDigits) + 'f', [Tmp]);
end;

procedure TfrmCalculator.btnSignClick(Sender: TObject);
var
  Tmp: Extended;
begin
  Tmp := StrToFloat(txtDisplay.Text);
  txtDisplay.Text := FloatToStr(-Tmp);
end;

function TfrmCalculator.GetValue: String;
begin
  Result := txtDisplay.Text;
end;

procedure TfrmCalculator.FormShow(Sender: TObject);
begin
  // force focus
  QWidget_setActiveWindow(Handle);
end;

procedure TfrmCalculator.SetValue(const Value: String);
var
  Tmp: Extended;
begin
  try
    Tmp := StrToFloat(Value);
  except
    Tmp := 0;
  end;
  txtDisplay.Text := FloatToStr(Tmp);
end;

end.
