{
TCalcEdit: a useful editbox with integrated calculator and
           currency converter.
           This component could be greatly enhanced...
           Code happily written with Borland Kylix.


Author: Giuseppe Madaffari
Copyright:  2000 by Giuseppe Madaffari. All rights reserved.
Created: 10/11/2000
Modified: 12/02/2001
Version: 1.0
License: you can freely use and distribute the included code
         for any purpouse, but you cannot remove this copyright
         notice. Send me any comment and update, they are really
         appreciated.
Contact: gmadaffari@infomedia.it
         www.programmers.net/artic/Madaffari/
         web.tiscalinet.it/gmadaffari
}

unit QCalcEdit;

interface

uses
  SysUtils, Types, Classes, Qt, QGraphics, QControls, QStdCtrls, QExtCtrls,
  QButtons, QForms;

type
  TCalcEdit = class(TFrameControl)
  private
    FEdit: TEdit;
    FButton: TBitBtn;
    FEnabled: Boolean;
    FCalc: TForm;
    FOnChange: TNotifyEvent;
    FOnReturnPressed: TNotifyEvent;
    FDecimalDigits: Integer;
    FConversionRatio: Single;
    FForeignCurSymbol: string;
    // Internal handlers
    procedure ButtonClick(Sender: TObject);
    procedure EditDblClick(Sender: TObject);

    // forwarded TEdit events' routines
    procedure passOnChange(Sender: TObject);
    procedure passOnClick(Sender: TObject);
    procedure passOnContextPopup(Sender: TObject; MousePos: TPoint;
      var Handled: Boolean);
    procedure passOnDblClick(Sender: TObject);
    procedure passOnEnter(Sender: TObject);
    procedure passOnExit(Sender: TObject);
    procedure passOnKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure passOnKeyPress(Sender: TObject; var Key: Char);
    procedure passOnKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure passOnMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure passOnMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure passOnMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure passOnReturnPressed(Sender: TObject);

    // property access routines
    procedure SetEnabled(const Value: boolean); reintroduce;
    procedure SetOnChange(const Value: TNotifyEvent);
    procedure SetOnReturnPressed(const Value: TNotifyEvent);
    procedure SetConversionRatio(const Value: Single);
    procedure SetDecimalDigits(const Value: Integer);
    procedure SetForeignCurSymbol(const Value: string);
    function passGetText: String;
    procedure passSetText(const Value: String);
  protected
    procedure ChangeBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    // new properties
    property ForeignCurSymbol: string read FForeignCurSymbol write SetForeignCurSymbol;
    property DecimalDigits: Integer read FDecimalDigits write SetDecimalDigits;
    property ConversionRatio: Single read FConversionRatio write SetConversionRatio;
    // forwarded properties
    property Text: String read passGetText write passSetText;
    property Enabled: boolean read fEnabled write SetEnabled default True;
    property TabStop;
    property TabOrder;
    // forwarded events
    property OnChange: TNotifyEvent read FOnChange write SetOnChange;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseUp;
    property OnClick;
    property OnDblClick;
    property OnContextPopup;
    property OnReturnPressed: TNotifyEvent read FOnReturnPressed write SetOnReturnPressed;
  end;

procedure Register;

implementation

uses
  Calc;

// contains the glyph for the speedbutton  
{$R 'CalcEdit.res'}

procedure Register;
begin
  RegisterComponents('Samples', [TCalcEdit]);
end;

{ TCalcEdit }

constructor TCalcEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Caption := '';
  TabStop := True;
  Height := 25;
  BorderStyle := bsSunken3D;

  // Create and setup the editbox
  FEdit := TEdit.Create(Self);
  with FEdit do
  begin
    Parent := Self;
    AutoSize := False;
    BorderStyle := bsNone;
    OnChange := passOnChange;
    OnClick := passOnClick;
    OnDblClick := EditDblClick;
    OnContextPopup := passOnContextPopup;
    OnEnter := passOnEnter;
    OnExit := passOnExit;
    OnKeyDown := passOnKeyDown;
    OnKeyPress := passOnKeyPress;
    OnKeyUp := passOnKeyUp;
    OnMouseDown := passOnMouseDown;
    OnMouseMove := passOnMouseMove;
    OnMouseUp := passOnMouseUp;
    OnReturnPressed := passOnReturnPressed;
  end;

  // Create and setup the button
  FButton := TBitBtn.Create(Self);
  with FButton do
  begin
    Parent:= Self;
    Caption := '';
    TabStop := False;
    OnClick := ButtonClick;
    // Now we have to load the small glyph: we should catch exception
    // otherwise, if the resource is missing the constructor fails
    // and could break the entire program!
    try
      Glyph := TBitmap.Create;
      // loaded from CalcEdit.res
      Glyph.LoadFromResourceName(Hinstance, 'DOWN');
    except
      // Ok, no glyph, free the bitmap
      Glyph.Free;
    end;
  end;

  // set frame's width = Edit+Button
  SetBounds(Left, Top, FEdit.Width + FButton.Width, Height);
  FEnabled := True;
end;

// Not so useful right now
destructor TCalcEdit.Destroy;
begin
  inherited;
end;

procedure TCalcEdit.SetEnabled(const Value: boolean);
begin
  if Enabled <> Value then
  begin
    FEnabled := Value;
    FEdit.Enabled := Value;
    FButton.Enabled := Value;
  end;
end;

procedure TCalcEdit.ButtonClick(Sender: TObject);
var
  OldColor: TColor;
begin
  OldColor := FEdit.Font.Color;
  // User selected the button, create a calculator
  with TfrmCalculator.Create(FEdit, FConversionRatio,
    FDecimalDigits, FForeignCurSymbol) do
  try
    // align it to the bottom right edge
    Left := Self.ClientOrigin.X + Self.Width - Width;
    Top := Self.ClientOrigin.Y + Self.Height;
    // set initial value from the Edit
    Value := FEdit.Text;
    // show simple help
    FEdit.Font.Color := clGray;
    FEdit.Text := '<Esc> to close';
    // Show the calendar and wait until user presses Esc
    ShowModal;
    // set focus (owned at this time by the button) back to the Edit
    FEdit.SetFocus;
  finally
    // restore color, set Edit.Text and free calculator
    FEdit.Font.Color := OldColor;
    FEdit.Text := Value;
    FCalc.Free;
  end;
end;

procedure TCalcEdit.EditDblClick(Sender: TObject);
begin
  // double click acts like selecting the button
  FButton.Click;
  passOnDblClick(Sender);
end;

procedure TCalcEdit.ChangeBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited;
  // the controls has been resized, we need to recalculate subcontrols' size
  if Assigned(FButton) and Assigned(FEdit) then
  begin
    // place the button to the rigth edge of the frame
    FButton.SetBounds(Width - Height - 1, 2, Height - 2, Height - 2);
    FEdit.SetBounds(2, 2, FButton.Left - 3, Height - 2);
  end;
end;

// The following methods don't do a lot of work, they simply pass
// events generated by UI and forward them to the Edit control

procedure TCalcEdit.passOnClick(Sender: TObject);
begin
  if Assigned(OnClick) then
    OnClick(Self);
end;

procedure TCalcEdit.passOnMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Assigned(OnMouseDown) then
    OnMouseDown(Self, Button, Shift, X, Y);
end;

procedure TCalcEdit.passOnMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Assigned(OnMouseUp) then
    OnMouseUp(Self, Button, Shift, X, Y);
end;

procedure TCalcEdit.passOnChange(Sender: TObject);
begin
  if Assigned(OnChange) then
    OnChange(Self);
end;

procedure TCalcEdit.passOnContextPopup(Sender: TObject; MousePos: TPoint;
  var Handled: Boolean);
begin
  if Assigned(OnContextPopup) then
    OnContextPopup(Self, MousePos, Handled);
end;

procedure TCalcEdit.passOnDblClick(Sender: TObject);
begin
  if Assigned(OnDblClick) then
    OnDblClick(Self);
end;

procedure TCalcEdit.passOnEnter(Sender: TObject);
begin
  if Assigned(OnEnter) then
    OnEnter(Self);
end;

procedure TCalcEdit.passOnExit(Sender: TObject);
begin
  if Assigned(OnExit) then
    OnExit(Self);
end;

procedure TCalcEdit.passOnKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Assigned(OnKeyDown) then
    OnKeyDown(Self, Key, Shift);
end;

procedure TCalcEdit.passOnKeyPress(Sender: TObject; var Key: Char);
begin
  if Assigned(OnKeyPress) then
    OnKeyPress(Self, Key);
end;

procedure TCalcEdit.passOnKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Assigned(OnKeyUp) then
    OnKeyUp(Self, Key, Shift);
end;

procedure TCalcEdit.passOnMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if Assigned(OnMouseMove) then
    OnMouseMove(Self, Shift, X, Y);
end;

procedure TCalcEdit.passOnReturnPressed(Sender: TObject);
begin
  if Assigned(OnReturnPressed) then
    OnReturnPressed(Self);
end;

function TCalcEdit.passGetText: String;
begin
  if Assigned(FEdit) then
    Result := FEdit.Text
  else
    Result := '';
end;

procedure TCalcEdit.passSetText(const Value: String);
begin
  if Assigned(FEdit) then
    FEdit.Text := Value;
end;

// Property setters

procedure TCalcEdit.SetOnChange(const Value: TNotifyEvent);
begin
  FOnChange := Value;
end;

procedure TCalcEdit.SetOnReturnPressed(const Value: TNotifyEvent);
begin
  FOnReturnPressed := Value;
end;

procedure TCalcEdit.SetConversionRatio(const Value: Single);
begin
  FConversionRatio := Value;
end;

procedure TCalcEdit.SetDecimalDigits(const Value: Integer);
begin
  FDecimalDigits := Value;
end;

procedure TCalcEdit.SetForeignCurSymbol(const Value: string);
begin
  FForeignCurSymbol := Value;
end;

end.
